/*
Copyright (C) 2011, 2012, 2013 Craciun Dan <craciun.dan@tuxarena.com>

This file is part of NeonView.

NeonView is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
(at your option) any later version.

NeonView is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with NeonView.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h> // fprintf()
#include <stdlib.h> // exit()
#include <string.h>

#include <dirent.h> // opendir()
#include <pwd.h> // getpwuid()
#include <sys/stat.h> // mkdir()
#include <sys/types.h> // mkdir(), getpwuid(), getuid()
#include <unistd.h> // getwd(), getuid()

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <gdk/gdkkeysyms-compat.h>

/* program info */
#define NAME "NeonView"
#define VERSION "0.8.0"
#define HTTP "http://www.tuxarena.com/neonview/"
#define CODENAME "Ancistrus"
#define AUTHOR "Craciun Dan"
#define MAIL "craciun.dan@tuxarena.com"
#define DATE "Feb 13, 2013"

#define PATH_LENGTH 4096 // TODO: pathnames in linux have no limit defined, implement this properly

typedef struct environment {
  char cwd[PATH_LENGTH + 1]; // current working directory - full path from which the program was launched
  char *home; // home dir - full path as returned by getpwuid()
  char *config_dir; // config dir - full path e.g. /home/$USER/.neonview
  char *config_file; // config file - full path e.g. /home/$USER/.neonview/neonview.cfg
} TENV;

typedef struct config {
  char last_dir[PATH_LENGTH + 1];
  char start_dir[PATH_LENGTH + 1];
  int window_width;
  int window_height;
  char start_maximized[4];
  char show_toolbar[4];
  char show_status[4];
  char show_left_sidebar[4];
  char show_right_sidebar[4];
  char remember_fit[4];
  char fit_next[4];
  char remember_zoom[4];
  char remember_window_size[4];
  char window_title[65];
  char gtk_win_pos[29];

  char tmp_start_maximized[4];
  char tmp_remember_window_size[4];
  char tmp_remember_fit[4];
  char tmp_fit_next[4];
} TCFG;

typedef struct image_info {
  char *image_path; // image path as the argument was passed e.g. neonview ../../dir/image.jpg
  char *image_dirname; // image dirname e.g. /home/user/dir
  char *image_basename; // image basename e.g. image.jpg
  char *image_full_path; // image full path e.g. /home/user/dir/image.jpg
  int image_width;
  int image_height;
  int image_width_orig;
  int image_height_orig;
  int isgif; // 1 if image is gif
  int isfit;

  char **image_list; // array of strings containing all the image files in a directory, unsorted
  char **image_list_name; // sorted by name
  char **image_list_type; // sorted by extension
  char **image_list_size; // sorted by size
  char **image_list_date; // sorted by date
  char **image_list_basename;
  int total; // total number of images in directory
  int pos; // position of current displayed image
  int it; // index for thumbnails in left sidebar
} TNFO;

typedef struct gtk {
  GtkWidget *window;
  GtkWidget *grid1, *grid11, *grid12, *grid13, *grid13a, *grid13b, *grid13c, *grid14;
  GtkWidget *menubar, *menu_prev, *menu_next;
  GtkWidget *toolbar;
  GtkToolItem *prev, *next;
  GtkWidget *scrolled;
  GtkWidget *notebook1, *notebook2; // notebooks for left and, respectively, right sidebar
  GtkWidget *label_right1, *label_right2; // info labels in right sidebar
  GtkWidget *entry21, *entry22, *entry23, *entry24; // entry for notebook2 on sidebar 2, first entry

  GdkPixbuf *pixbuf;
  GdkPixbuf *pixbuf_orig; // will keep a copy of the original image
  GdkPixbuf *pixbuf_next; // preload next
  GdkPixbuf *pixbuf_prev; // preload prev
  GtkWidget *image; // current image
  GtkWidget *image_next; // preload next
  GtkWidget *image_prev; // preload prev

  GdkPixbufAnimation *animation;
  GdkPixbufAnimation *animation_orig;

  GtkWidget **button;

  GtkWidget *window_settings;

  GtkWidget *status1, *status2, *status3, *status4;
} TGTK;

typedef struct master {
  int argc; // number of command-line arguments
  char **argv; // array of strings containing command-line arguments

  TENV *tenv;
  TCFG *tcfg;
  TNFO *tnfo;
  TGTK *tgtk;
} TM;

/***** init.c *****/
void criterr (unsigned short); // display an error message
void init_args (TM *); // data initialization

void get_cwd (TM *); // get current working directory
void get_home (TM *); // get home directory for the current user
void get_config_dir (TM *); // get config directory as string
void get_config_file (TM *); // get config file as string

void read_config (TM *); // read config file
void write_default_config (TM *); // write default config file
void load_default_config (TM *); // load default config values
void write_config (TM *); // write config file with current values
void get_config (TM *); // get current config values - used by write_config()
void gtk_neon_quit (TM *);

/***** utils.c *****/
void remove_spaces (char *); // remove all blanks in a string, modifes initial string
void assign_keys (TM *, char *, char *); // assign key = val pairs to struct config members
void print_config (TM *); // print config values in struct config
void check_keys (TM *); // check to see if all keys are filled in appropriately
void init_keys (TM *); // initialize all keys to NULL
int get_win_pos (char *); // translate tm->tcfg->gtk_win_pos into an integer to use with gtk_window_set_position()
void init_struct (TM *); // initialize struct vals with NULL
int check_ext (char *); // check if a filename is an image
int fisfile (char *); // check if a file can be opened
void get_full_path (TM *);
void get_dirname (TM *);

/***** stdc.c *****/
char *lc (char *); // convert a string to lowercase
char *rts (char *); // reverse string
int all_digits (char *); // check if a string contains only digits
long filesize (char *); // return the size of a file in bytes
int perms (char *); // 1000 - exists, 1100 r--, 1110 rw-, 1111 rwx
int digits (int);

/***** check.c *****/
void check (TM *); // function to test various stuff, not pertinent to the program

/***** widgets.c *****/
void put_window (TM *);
void put_menubar (TM *);
void put_toolbar (TM *);
void put_scrolled (TM *);
void put_status (TM *);
void put_left_sidebar (TM *);
void put_right_sidebar (TM *);
void toggle_toolbar (GtkWidget *, TM *);
void toggle_status (GtkWidget *, TM *);
void toggle_left_sidebar (GtkWidget *, TM *);
void toggle_right_sidebar (GtkWidget *, TM *);
void destroy_status (TM *);

/***** image.c *****/
void show_image (TM *);
void zoom_in_image (TM *);
void zoom_out_image (TM *);
void rotate_left_image (TM *);
void rotate_right_image (TM *);
void flip_image (TM *);
void fill_nfo (TM *);
void load_orig_image (TM *);
void fill_image_list (TM *);
void show_prev (TM *);
void show_next (TM *);
void fill_right_sidebar (TM *);
void thumb_show (TM *, GtkWidget *);
void fit_to_window (TM *);

/***** settings.c *****/
void put_settings (TM *);
void show_hint00 (GtkWidget *);
void show_hint11 (GtkWidget *);
void show_hint12 (GtkWidget *);
void show_hint21 (GtkWidget *);
void show_hint22 (GtkWidget *);
void toggle_start_maximized (TM *, GtkWidget *);
void toggle_remember_window_size (TM *, GtkWidget *);
void toggle_remember_fit (TM *, GtkWidget *);
void toggle_fit_next (TM *, GtkWidget *);
void settings_ok (TM *);
void settings_cancel (TM *);

/***** help.c *****/
void put_manual ();
void put_faq ();
void put_homepage ();
void put_about ();
void about_ok (GtkWidget *);

/***** open.c *****/
void put_open (TM *);
void put_save (TM *); // to be implemented
void put_save_as (TM *); // to be implemented
